//---------------------------------------------------------------------------
#include <vcl.h>
#include <math.h>
#include <stdio.h>
#pragma hdrstop

#include "Main.h"
#include "Pots.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMain *frmMain;
//---------------------------------------------------------------------------
const double PI = 3.14156;
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
    : TForm(Owner), Inited(false), ThePot(0) {
}
//---------------------------------------------------------------------------
double 
TfrmMain::CalcFreq(double r, double c) {
    return 1.0 / (2.0 * PI * r * c * 0.001); // * = 1000 * 0.000 001 = 0.001
}
//---------------------------------------------------------------------------
double
TfrmMain::CalcRes(double f, double c) {
    return 1.0 / (2.0 * PI * f * c * 0.001);
}
//---------------------------------------------------------------------------
double
TfrmMain::CalcFreq(double da) {
    double vrr = GetPot()->GetResistance(da);
    double r = vrr + AddR;
    return CalcFreq(r, C);
}
//---------------------------------------------------------------------------
double
TfrmMain::GetStep(double Range) {
	const int size = 3;
    int k[size] = {1, 2, 5};
    if (Range < 10)
    	throw Exception("    10");
    for (int m = 1; m < 1000000; m *= 10)
    	for (int i = 0; i < size; i++) {
        	double s = m*k[i];
            double cnt = Range / s;
            if (cnt >= 10 && cnt <= 25)
            	return s; 
        }
    throw Exception("  ");
}
//---------------------------------------------------------------------------
double
TfrmMain::Round(double Value, double Precision) {
	int v = ((int)Value) / ((int)Precision);
    v *= (int)Precision;
    if (v < Value)
    	v += (int)Precision;
    return v;
}
//---------------------------------------------------------------------------
void
__fastcall TfrmMain::pbScalePaint(TObject *Sender) {
    GetParams();
	if (!Inited)
    	return;

    int w = pbScale->Width;
    int h = pbScale->Height;
    if (w < h)
        h = w;
    else
        w = h;
    pbScale->Canvas->Pen->Color = clBlack;
    pbScale->Canvas->Pen->Style = psSolid;
    pbScale->Canvas->Brush->Color = clWhite;
    pbScale->Canvas->Brush->Style = bsSolid;
    pbScale->Canvas->Rectangle(w / 8, h / 8, w - w / 8 - 1, h - h / 8 - 1);

    int dx = (w / 4)*sin(StartAngle*PI / 180.0);
    int dy = (h / 4)*cos(StartAngle*PI / 180.0);

    pbScale->Canvas->Arc(w / 4, h / 4, 3*w / 4, 3*h / 4,
                         w / 2 + dx, h / 2 + dy,
                         w / 2 - dx, h / 2 + dy);

	double fmin = CalcFreq(FullAngle);
    double fmax = CalcFreq(0);
    double step = GetStep(fmax - fmin);
    double fstart = Round(fmin, step);

    DrawFreq(w, h, fmin, true);
    DrawFreq(w, h, fmax, true);
    for (double f = fstart; f <= fmax; f += step) 	//      step
        DrawFreq(w, h, f);                          // " ",   
}
//---------------------------------------------------------------------------
void
TfrmMain::DrawFreq(int w, int h, double f, bool hide_text) {
    double r = CalcRes(f, C);                   // ,      
    int a = GetPot()->GetAngle(r - AddR);       //       
    int da = - (StartAngle + a);     			//   "" 
    DrawMark(w, h, da);
    if (!hide_text)
    	DrawText(w, h, da, (int)f);
}
//---------------------------------------------------------------------------
void
TfrmMain::DrawMark(int w, int h, double da) {
    int x1 = (w / 4)*sin(da*PI / 180.0);
    int y1 = (h / 4)*cos(da*PI / 180.0);
    int x2 = (w / 4 + 10)*sin(da*PI / 180.0);
    int y2 = (h / 4 + 10)*cos(da*PI / 180.0);
    pbScale->Canvas->MoveTo(w / 2 + x1, h / 2 + y1);
    pbScale->Canvas->LineTo(w / 2 + x2, h / 2 + y2);
}
//---------------------------------------------------------------------------
void
TfrmMain::DrawText(int w, int h, double da, AnsiString txt) {
    int th = pbScale->Canvas->TextHeight(txt);
    int tw = pbScale->Canvas->TextWidth(txt);
    int x3 = (w / 4 + 20)*sin(da*PI / 180.0);
    int y3 = (h / 4 + 20)*cos(da*PI / 180.0);
    pbScale->Canvas->TextOut(w / 2 + x3 - tw / 2, h / 2 + y3 - th / 2, txt);
}
//---------------------------------------------------------------------------
void
TfrmMain::GetParams() {
	try {
	    FullAngle = edAngle->Text.ToInt();
        StartAngle = (360 - FullAngle) / 2;
	    VarR = edVarR->Text.ToDouble(); // 
    	AddR = edR->Text.ToDouble();    // 
	    C = edC->Text.ToDouble();     // 
    }
    catch (...) {
	    FullAngle = 0;
        StartAngle = 0;
	    VarR = 0;
    	AddR = 0;
	    C = 0;
    }

    Inited = (FullAngle > 0) && (VarR > 0) && (AddR > 0) && (C > 0);

    if (ThePot)
    	delete ThePot;
    switch (cbChrt->ItemIndex) {
    	case 0: ThePot = new Pot(VarR, FullAngle); break;
        case 1: ThePot = new LogPot(VarR, FullAngle); break;
        case 2: ThePot = new LogPot(VarR, FullAngle); break;
        default: ThePot = 0; break;
    }
}
//---------------------------------------------------------------------------
Pot *
TfrmMain::GetPot() {
	return ThePot;
}
//---------------------------------------------------------------------------
void
__fastcall TfrmMain::ParamsChanged(TObject *Sender) {
	GetParams();
    pbScale->Invalidate();
    pbGraph->Invalidate();
}
//---------------------------------------------------------------------------
void
__fastcall TfrmMain::pbGraphPaint(TObject *Sender) {
    GetParams();
	if (!Inited)
    	return;

    int w = pbGraph->Width;
    int h = pbGraph->Height;
    pbGraph->Canvas->Pen->Color = clBlack;
    pbGraph->Canvas->Pen->Style = psSolid;
    pbGraph->Canvas->Brush->Color = clWhite;
    pbGraph->Canvas->Brush->Style = bsSolid;
    pbGraph->Canvas->Rectangle(0, 0, w - 1, h - 1);

    pbGraph->Canvas->MoveTo(0, h - 1);

    for (int i = 0; i <= FullAngle; i++) {
        double prc = (double)i / FullAngle;
        double vrr = GetPot()->GetResistance(i);
        double fr = GetPot()->GetMaxResistance();
        pbGraph->Canvas->LineTo(prc*w, h - (vrr / fr)*h - 1);
    }
}
//---------------------------------------------------------------------------

