//---------------------------------------------------------------------------
#include "Pots.h"
//---------------------------------------------------------------------------
double
PotPoint::GetCoeff(const PotPoint &Next) {
    return (Next.Resistance - Resistance) / (Next.AnglePrc - AnglePrc);
}
//---------------------------------------------------------------------------
double
PotPoint::operator/(const PotPoint &Next) {
    return GetCoeff(Next);
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
void
PotTbl::AddPoint(PotPoint p) {
      Tbl.push_back(p);
}
//---------------------------------------------------------------------------
PotPoint
PotTbl::GetPoint(int Idx) {
    if (Idx != -1)
            return Tbl[Idx];
    throw "Unknown point (idx = -1)";
}
//---------------------------------------------------------------------------
double
PotTbl::GetResPrc(double AnglePrc) {
    int idx = -1;
    for (size_t i = 0; i < Tbl.size() - 1; i++)
        if (Tbl[i].AnglePrc <= AnglePrc && AnglePrc <= Tbl[i + 1].AnglePrc)
            idx = i;
    PotPoint bp = GetPoint(idx);
    PotPoint ep = GetPoint(idx + 1);
    return (bp.Resistance + (AnglePrc - bp.AnglePrc)*(bp / ep));
}
//---------------------------------------------------------------------------
double
PotTbl::GetAnglePrc(double ResPrc) {
    int idx = -1;
    for (size_t i = 0; i < Tbl.size() - 1; i++)
        if (Tbl[i].Resistance <= ResPrc && ResPrc <= Tbl[i + 1].Resistance)
            idx = i;
    PotPoint bp = GetPoint(idx);
    PotPoint ep = GetPoint(idx + 1);
    return (bp.AnglePrc + (ResPrc - bp.Resistance) / (bp / ep));
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
LogPotTbl::LogPotTbl() {
        const int size = 7;
        PotPoint LogTbl[size] = {
            {0.000, 0.0}, {0.034, 0.2}, {0.093, 0.4}, {0.200, 0.6},
            {0.450, 0.8}, {0.650, 0.9}, {1.000, 1.0}
        };
        for (int i = 0; i < size; i++)
                AddPoint(LogTbl[i]);
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
Pot::Pot(double AResistance)
    : Resistance(AResistance), FullAngle(270) {
}
//---------------------------------------------------------------------------
Pot::Pot(double AResistance, int AFullAngle)
    : Resistance(AResistance), FullAngle(AFullAngle) {
}
//---------------------------------------------------------------------------
double
Pot::GetResPrc(double Angle) {
    double res = Angle;
    res /= FullAngle;
    if (res > 1.0)
        res = 1.0;
    return res;
}
//---------------------------------------------------------------------------
double
Pot::GetResistance(double Angle) {
    return Resistance*GetResPrc(Angle);
}
//---------------------------------------------------------------------------
double
Pot::GetAnglePrc(double Res) {
    double res = Res;
    res /= Resistance;
    if (res > 1.0)
        res = 1.0;
    return res;
}
//---------------------------------------------------------------------------
double 
Pot::GetAngle(double Res) {
	return FullAngle*GetAnglePrc(Res);
}
//---------------------------------------------------------------------------
double
Pot::GetMaxResistance() {
    return GetResistance(FullAngle);
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
LogPot::LogPot(double AResistance)
    : inherited(AResistance) {
}
//---------------------------------------------------------------------------
LogPot::LogPot(double AResistance, int AFullAngle)
    : inherited(AResistance, AFullAngle) {
}
//---------------------------------------------------------------------------
double
LogPot::GetResPrc(double Angle) {
    return Tbl.GetResPrc(inherited::GetResPrc(Angle));
}
//---------------------------------------------------------------------------
double
LogPot::GetAnglePrc(double Res) {
	return Tbl.GetAnglePrc(inherited::GetAnglePrc(Res));
}
//---------------------------------------------------------------------------

